
source("fish_corefunc.r")

# simulates the population dynamics over time
simulate <- function(
  ind,                 # initial population (matrix with age and weight)
  years,               # number of years to simulate (integer)
  pars,                # parameters, possibly passed on to functions
  statfun              # function to compute annual summary statistics
) {

  # initialize summary variable
  s <- cbind(year=0, statfun(ind))

  # loop over years
  for (y in 1:years) {
    
    # aging
    ind[,"age"] <- ind[,"age"] + 1

    # somatic growth, considering carrying capacity
    ind[,"weight"] <- ind[,"weight"] + 
      growth(weight=ind[,"weight"], k.grw.max=pars["k.grw.max"],
      m.grw.half=pars["m.grw.half"], capacity=pars["capacity"],
      rand=subopt(pars["beta.grw"]))

    # natural mortality
    alive <- survived(n=nrow(ind), p.surv.max=pars["p.surv.max"],
      rand=subopt(pars["beta.surv"]))
    ind <- ind[alive,,drop=F]
    if (nrow(ind) == 0)  break
    
    # anthropogenic mortality (catch)
    caught <- fished(weight=ind[,"weight"],
      catch.minweight=pars["catch.minweight"],
      catch.fraction=pars["catch.fraction"])
    ind <- ind[!caught,,drop=F]
    if (nrow(ind) == 0)  break

    # reproduction
    indiv.eggs <- eggs(age=ind[,"age"], weight=ind[,"weight"],
      k.repro.max=pars["k.repro.max"], a.repro=pars["a.repro"],
      rand=subopt(pars["beta.repro"]))
    total.eggs <- sum(indiv.eggs)

    # add offspring after considering mortality
    off <- offspr(eggs=total.eggs, p.surv.max.off=pars["p.surv.max.off"],
      weight.zero=pars["weight.zero"], rand=subopt(pars["beta.off"]))
    ind <- rbind(ind, off)

    # record statistics for current year
    s <- rbind(s, cbind(year=y, statfun(ind)))
  }
  
  return(s)
}
