
# Function to account for inter-annual variations in rates. Returns
# a random number in range 0...1 to be used as a multiplier for reducing
# maximum rates to their actual annual values in a stochastic manner.

subopt <- function(
  beta    # numeric in 0...1, controls the result distribution
) {
  runif(1)^beta
}

########################################################################

# Returns the annual gain in individual weights (numeric vector)
# as a result of somatic growth (kg).

growth <- function(
  weight,       # numeric vector of individual weights (kg)
  k.grw.max,    # max. somatic growth rate (1/a)
  m.grw.half,   # controls slow-down of growth with weight (kg)
  capacity,     # carrying capacity (kg)
  rand          # random number in 0...1; stochastic effects 
) {
  rand * k.grw.max * weight *
    m.grw.half / (m.grw.half + weight) *
    max(0, 1 - sum(weight) / capacity)
}

########################################################################

# Returns a vector with the number of eggs per individuals.

eggs <- function(
  age,         # vector of ages of individuals (years)
  weight,      # vector of weights of individuals (kg)
  k.repro.max, # max. reproduction rate (eggs / kg)
  a.repro,     # minimum age of reproduction (years)
  rand         # random number in 0...1; stochastic effects 
) {
  floor(rand * k.repro.max * weight * as.numeric(age >= a.repro))
}

########################################################################

# Translates the number of eggs into population offspring. Returns a
# matrix of new individuals.
#
#   eggs: Number of eggs
#   beta: controls annual stochastic effect (-), ineffective if 0

offspr <- function(
  eggs,           # total number of eggs (scalar integer)
  p.surv.max.off, # max. survival prob. of zero-age individuals (0...1)
  weight.zero,    # weight at zero age (kg)
  rand            # random number in 0...1; stochastic effects 
) {
  n <- floor(eggs * p.surv.max.off * rand)
  cbind(age=rep(0, n), weight=rep(weight.zero, n))
}

########################################################################

# Returns a logical vector indicating survived individuals.
#
#   n: population size (integer)
#   beta: controls annual stochastic effect (-), ineffective if 0

survived <- function(
  n,             # number of individuals (integer)
  p.surv.max,    # max. annual survival probability (0...1)
  rand           # random number in 0...1; stochastic effects 
) {
  runif(n) < (p.surv.max * rand)
}

########################################################################

# Returns an logical vector, representing individuals lost to fishing.

fished <- function(
  weight,            # weights of individuals (numeric vector)
  catch.minweight,   # minimum weight (numeric)
  catch.fraction     # fraction above min. weight being caught (numeric)
) {
  i <- which(weight >= catch.minweight)
  if (length(i) > 0) {
    i <- i[0:floor(catch.fraction * length(i))]
  }
  caught <- rep(FALSE, length(weight))
  if (length(i) > 0) {
    caught[i] <- TRUE
  }
  caught
}

