
# computes statistics for all properties of the population,
# input: matrix with individuals in rows, properties in columns
# output: data frame with columns "variable", "stats", "value"
pop.stats <- function(x) {
  st <- c("min","q1","median","mean","q3","max")
  s <- NULL
  for (n in colnames(x)) {
    s <- rbind(s, data.frame(variable=n, stats="min", value=min(x[,n])))
    s <- rbind(s, data.frame(variable=n, stats="max", value=max(x[,n])))
    s <- rbind(s, data.frame(variable=n, stats="mean", value=mean(x[,n])))
    for (p in c("05","10","25","50","75","90","95")) {
      s <- rbind(s, data.frame(variable=n, stats=paste0("p",p),
        value=unname(quantile(x[,n], probs=as.numeric(p)/100))))
    }
    # weight where 50% of total bioweight is larger / smaller
    m <- sort(x[,"weight"])
    i <- which.min((cumsum(m) - 0.5*sum(m))^2)
    s <- rbind(s, data.frame(variable=n, stats="biom50", value=x[[i,n]]))
  }
  s
}

# plot quantiles of population properties over time
# input: output of "pop.stats" with a "year" column appended
plot.over.time <- function(x, v, log, ylab=v) {
  x <- x[x[,"variable"] == v, colnames(x) != "variable"]
  layout(matrix(1:2, ncol=2), widths=c(0.75, 0.25))
  plot(range(x[,"year"]), range(x[,"value"], na.rm=T),
    type="n", xlab="Year", ylab=ylab, log=log)
  x <- reshape2::dcast(year ~ stats, data=x, value.var="value")
  yrs <- c(x[,"year"], rev(x[,"year"]))
  polygon(x=yrs, y=c(x[,"min"], rev(x[,"max"])), col="slategray1", border=NA)
  polygon(x=yrs, y=c(x[,"p05"], rev(x[,"p95"])), col="slategray2", border=NA)
  polygon(x=yrs, y=c(x[,"p10"], rev(x[,"p90"])), col="slategray3", border=NA)
  polygon(x=yrs, y=c(x[,"p25"], rev(x[,"p75"])), col="slategray4", border=NA)
  lines(x[,"year"], x[,"p50"])
  lines(x[,"year"], x[,"mean"], lty=2)
  # legend
  omar <- par("mar")
  par(mar=rep(0.1, 4))
  plot(0, 0, type="n", ann=F, axes=F)
  legend("left", bty="n", fill=c("slategray1", "slategray2",
    "slategray3", "slategray4", "transparent", "transparent"),
    border=c("slategray1", "slategray2",
    "slategray3", "slategray4", "transparent", "transparent"),
    lty=c(rep(NA, 4), 1, 2),
    legend=c("Range","5-95%","10-90%","IQR","Median","Mean"))
  par(mar=omar)
  layout(1)
}
